import re
import os
import subprocess

from agent.common.pylog import functionLog, logger


@functionLog
def sysCommand(command: str, cwd: str = "./"):
    '''Run system command with subprocess.run and return result
    '''
    logger.info(command)
    result = subprocess.run(
        command,
        shell=True,
        close_fds=True,
        cwd=cwd,
        stderr=subprocess.PIPE,
        stdout=subprocess.PIPE
    )

    suc = (result.returncode == 0)
    out = result.stdout.decode('UTF-8', 'strict').strip()
    error = result.stderr.decode('UTF-8', 'strict').strip()

    if not suc:
        logger.error(error)
        return suc, error
    else:
        return suc, out
        
        
def _getNetQueue(dev_name):
    """ Get rx_queue and tx_queue

    Args:
        dev_name (string): net dev name
    """
    suc, _pci_number = sysCommand("ethtool -i {}".format(dev_name))
    if not suc:
        return None
    
    bus_info = re.search(r"bus-info: (.*)\n", _pci_number).group(1)
    if bus_info == '':
        return None

    suc, queue_info = sysCommand("ls /sys/class/net/{dev_name}/queues/".format(dev_name = dev_name))
    if not suc:
        return None

    rx_queue, tx_queue = re.findall(r"rx-\d",queue_info), re.findall(r"tx-\d",queue_info)
    return bus_info, rx_queue, tx_queue
    

def _getPCIDev():
    suc, dev_list = sysCommand("ls /sys/class/net")
    if not suc:
        return False, "Can not read dev list: {}".format(dev_list)

    PCI_dev_list = []
    for dev in dev_list.split("\n"):
        _path = os.path.join("/sys/class/net", dev)
        if not os.path.isdir(_path):
            continue
        try:
            _link_path = os.readlink(_path)
            if re.search(r"pci\d+", _link_path):
                if re.search(r"virtio\d", _link_path):
                    PCI_dev_list.append((re.search(r"virtio\d", _link_path).group(0), dev))
                else:
                    PCI_dev_list.append((dev,dev))
                    
        except OSError:
            continue
    
    return PCI_dev_list


def _getInterrupts(dev_name):
    def _getInterruptsCode(interrupts):
        result = []
        for i in interrupts:
            interrupts_id = re.search(r"^(\d+):", i).group(1)
            result.append(interrupts_id)
        return result
    
    suc, content = sysCommand("cat /proc/interrupts")
    if not suc:
        return None 
    content = content.split('\n')
    content = [i.strip() for i in content]
    
    input_output_interrupts = [i for i in content if re.search("{dev_name}-input|{dev_name}-output".format(dev_name = dev_name),i)]
    
    if not input_output_interrupts.__len__() == 0:
        return _getInterruptsCode(input_output_interrupts)
    
    interrupts = [i for i in content if re.search("{}".format(dev_name),i)]
    return _getInterruptsCode(interrupts)


def getNetInfo():
    PCI_dev_list = _getPCIDev()
    
    for virtio_name, dev_name in PCI_dev_list:
        res = _getNetQueue(dev_name = dev_name)
        if res:
            bus_info, rx_queue, tx_queue = res
            
        interrupts_queue_id = _getInterrupts(dev_name = virtio_name)
        return dev_name, bus_info, interrupts_queue_id, rx_queue, tx_queue

    return


def getNUMA(bus_info):
    suc, numa_message = sysCommand("lspci -vvvs {bus_info}".format(bus_info = bus_info))
    if not suc:
        return 
    
    if re.search(r"NUMA node: (\d)\n", numa_message):
        numa_node_num = re.search(r"NUMA node: (\d)\n", numa_message).group(1)
    else:
        numa_node_num = 0
    
    _, cpu_message = sysCommand("lscpu")
    if re.search(r"NUMA node{} CPU\(s\):\s*(\d+-\d+)".format(numa_node_num),cpu_message):
        numa_node_core_range = re.search(
            r"NUMA node{} CPU\(s\):\s*(\d+-\d+)".format(numa_node_num),cpu_message).group(1).split('-')
    elif re.search(r"NUMA 节点{} CPU：\s*(\d+-\d+)".format(numa_node_num),cpu_message):
        numa_node_core_range = re.search(
            r"NUMA 节点{} CPU：\s*(\d+-\d+)".format(numa_node_num),cpu_message).group(1).split('-')
    else:
        return
    
    numa_node_core_range = [int(i) for i in numa_node_core_range]
    return numa_node_core_range


def getCPUInfo():
    _, cpu_num   = sysCommand("cat /proc/cpuinfo| grep 'physical id'| sort| uniq| wc -l")
    _, processor = sysCommand("cat /proc/cpuinfo| grep 'processor'| wc -l")
    return int(cpu_num), int(processor)


if __name__ == "__main__":
    dev_name, bus_info, interrupts_queue, rx_queue, tx_queue = getNetInfo()
    print("dev_name:", dev_name)
    print("bus_info:", bus_info)
    print("interrupts_queue:", interrupts_queue)
    print("rx_queue:", rx_queue)
    print("tx_queue:", tx_queue)

    numa_node_core_range = getNUMA(bus_info)
    print("numa_node_core_range: ", numa_node_core_range)

    cpu_num, processor = getCPUInfo()
    print("cpu_num:", cpu_num)
    print("processor:", processor)