import os
import time

from agent.common.config import Config
from agent.common.system import sysCommand
from collections import defaultdict

from agent.common.pylog import logger

LIMITS_CONF = "/etc/security/limits.conf"
FILE_MAX = "/proc/sys/fs/file-max"


class Limits:
    def __init__(self):
        super().__init__()
        self.backup_file = os.path.join(Config.BACKUP_PATH, "limits.txt")

    def _getLimitsCmd(self, param_name, param_value):
        param = " ".join(param_name.split("_"))
        other_user = "* {}".format(param)
        root_user = "root {}".format(param)
        base_cmd = 'sed -i "/{0}/d" /etc/security/limits.conf;echo "{0} {1}" >> /etc/security/limits.conf'
        root_cmd = base_cmd.format(root_user, param_value)
        other_cmd = base_cmd.format(other_user, param_value)
        return """{};{}""".format(root_cmd, other_cmd)

    def _setLimitsFiles(self, param_name, param_value):
        _value = int(param_value)
        if "nofile" in param_name:
            cmd = self._getLimitsCmd(param_name, _value)
        elif param_name == "ulimit":
            cmd = "ulimit -n {}".format(_value)
        elif param_name == "file-max":
            cmd = "echo {} > {}".format(_value, FILE_MAX)
        return sysCommand(cmd)

    def _getParam(self, param_name):
        if "nofile" in param_name:
            param = " ".join(param_name.split("_"))
            cmd = "cat /etc/security/limits.conf |grep 'root {}'|awk -F' ' '{print $4}'".format(param)
        elif param_name == "ulimit":
            cmd = "ulimit -n"
        elif param_name == "file-max":
            cmd = 'cat {}'.format(FILE_MAX)
        return sysCommand(cmd)

    def setParamAll(self, param_list: dict):
        result = defaultdict(dict)
        success = False
        for param_name, param_info in param_list.items():
            _suc, _res = self._setLimitsFiles(param_name, param_info['value'])
            result[param_name] = {
                "value" : param_info['value'],
                "dtype" : param_info["dtype"],
                "suc"   : _suc,
                "msg"   : _res
            }
            success = success | _suc
        return success, result

    def getParamAll(self, param_list: dict):
        result = defaultdict(dict)
        for param_name, param_info in param_list.items():
            suc, param_value = self._getParam(param_name)
            result[param_name] = {
                "value" : param_value,
                "dtype" : param_info["dtype"],
                "suc"   : suc,
                "msg"   : ""
            }
        return True, result

    def rollback(self):
        if not os.path.exists(self.backup_file):
            return True, "Can not find backup file:{}".format(self.backup_file)

        cmd = "echo y|cp {} {}".format(os.path.join(Config.BACKUP_PATH, "limits.conf"), LIMITS_CONF)
        suc, res = sysCommand(cmd)
        if not suc:
            return False, "rollback limits conf failed: {}".format(res)

        with open(self.backup_file, 'r') as f:
            value = f.read()
        ulimit_value = value.split("\n")[0].split(": ")[1]
        file_max = value.split("\n")[1].split(": ")[1]
        suc, res = sysCommand("ulimit -n {}".format(ulimit_value))
        if not suc:
            return False, "rollback ulimit value failed: {}".format(res)
        suc, res = sysCommand("echo {} > {}".format(file_max, FILE_MAX))
        if not suc:
            return False, "rollback file_max failed: {}".format(res)
        os.remove(self.backup_file)
        os.remove(os.path.join(Config.BACKUP_PATH, "limits.conf"))
        return True, "rollback limits successfully!" 

    def backup(self, param_list: dict):
        if os.path.exists(self.backup_file):
            suc, res = self.rollback()
            if not suc:
                return False, res

        cmd = "echo y|cp {} {}".format(LIMITS_CONF, os.path.join(Config.BACKUP_PATH, "limits.conf"))
        suc, res = sysCommand(cmd)
        if not suc:
            return False, "backup limits conf failed: {}".format(res)
        status1, ulimit_value = sysCommand("ulimit -n")
        status2, file_max = sysCommand("cat {}".format(FILE_MAX))
        if status1 and status2:
            res = "ulimit: {}\nfile_max: {}".format(ulimit_value, file_max)
            with open(self.backup_file, 'w') as f:
                f.write(res)
            return True, self.backup_file
        else:
            return False, "backup limits value failed: {};{}".format(ulimit_value, file_max)


if __name__ == "__main__":
    limits = Limits()
    print(limits.setParamAll({"hard_nofile": {'value': 1048576, 'dtype': 'int'},
                              "soft_nofile": {'value': 1048576, 'dtype': 'int'},
                              "ulimit": {'value': 1048576, 'dtype': 'int'},
                              "file-max": {'value': 1048576, 'dtype': 'int'}}))
